<?php
namespace PrestaShop\Module\LCVPrestaConnector;

use LCVPrestaConnector;

/**
 * Configuration du module
 */
class SyncConfiguration
{
    /**
     * Préfixe des tables
     */
    private string $tblPrefix;

    /**
     * Cache de configuration
     */
    private $configurationCache = [];

    /**
     * Indique si la configuration a été pré-chargée
     */
    private $preLoaded = false;

    /**
     * Instanciation de la configuration
     */
    public function __construct(LCVPrestaConnector $module)
    {
       $this->tblPrefix = $module->TblPrefix; 
    }

    /**
     * Retourne vrai si la configuration a été pré-chargée, faux autrement
     */
    public function hasBeenPreloaded() : bool
    {
        return $this->preLoaded;
    }

    /**
     * Pré-charge la totalité de la configuration en mémoire de sorte
     * que les appels successifs soient moins couteux
     * 
     * @param array|null $keys Liste des clés à précharger, ou null pour tout précharger
     */
    public function preload(?array $keys = null)
    {     
        // Obtention de l'instance de base de données
        $db = \Db::getInstance();

        // Configuration
        $this->configurationCache = [];

        // Récupération de la configuration
        $query = 'SELECT * FROM ' . $this->tblPrefix . 'configuration';
        if ($keys) {
            $query .= ' WHERE `key` IN (';
            $query .= implode(',', array_map(function($key) {
                return '\'' . $key . '\'';
            }, $keys));
            $query .= ')';
        }        
        $result = $db->executeS($query);

        if ($result === false)
            throw new \Exception('Impossible de récupérer la configuration');
        
        // On parcourt les résultats et on les met en cache
        foreach ($result as $row) {
            $this->configurationCache[$row['key']] = json_decode($row['value'], true);
        }

        if (!$keys) {
            $this->preLoaded = true;
        }
    }

    /**
     * Obtient la valeur de configuration pour une clé donnée
     * ou null si la clé n'existe pas ou la valeur non positionnée.
     * 
     * Le premier appel est couteux, mais le résultat est gardé et entretenu en mémoire,
     * l'appel intensif sur la même valeur est donc sans conséquence.
     */
    public function get(string $key)
    {
        // Si le cache ne contient pas notre valeur, on la charge
        if (!isset($this->configurationCache[$key])) {
            // Obtention de l'instance de base de données
            $db = \Db::getInstance();

            // On cherche la valeur dans la base de données            
            $result = $db->executeS('SELECT `value` FROM ' . $this->tblPrefix . 'configuration WHERE `key` = \'' . $key . '\'');

            if ($result === false)
                throw new \Exception('Impossible de récupérer la configuration');                

            // On stocke la valeur si on l'a trouvé sinon, on stock null
            $this->configurationCache[$key] = count($result) > 0 ? 
                json_decode($result[0]['value'], true) : null;
        }

        // Puis on renvoi la valeur
        return $this->configurationCache[$key]; 
    }

    /**
     * Définit la valeur de configuration pour une clé donnée
     */
    public function set(string $key, array|string|float|int|bool $value)
    {
        // Obtention de l'instance de base de données
        $db = \Db::getInstance();

        // On met à jour le cache
        $this->configurationCache[$key] = $value;

        $type = gettype($value);

        // Sérialisation de la valeur
        $value = json_encode($value);

        // Puis on stock dans la base de données soit en insertion, soit en update si elle existe déjà
        // sans oublier d'échapper les valeurs pour éviter les injections SQL
        $value = $db->escape($value);

        if (!$db->execute('INSERT INTO ' . $this->tblPrefix . 'configuration (`key`, `value`) VALUES (\'' . $key . '\', \'' . $value . '\') ON DUPLICATE KEY UPDATE `value` = \'' . $value . '\''))
            throw new \Exception('Impossible de mettre à jour la configuration');
    }
}